/*
	
	vidview.cpp
	
	Copyright 1997 Be Incorporated, All Rights Reserved.
	
*/

#include <OS.h>
#include <signal.h>
#include "opt.h"
#include "vidview.h"
#include "vidsource.h"
#include "ControlWindow.h"

volatile int 	drawer_quit;		
thread_id       drawer_id;

double			numworker,numdrawer,numtimeout;
double			starttime,endtime;

static long 			__drawer(void *arg);

VideoView::VideoView(BRect rect, char *name)
	   	   : BView(rect, name, B_FOLLOW_ALL, B_WILL_DRAW)
{
	drawer_quit  = 0;
	numworker=numdrawer=numtimeout=0;
	
	fVideoSource = new BTSVideoSource("hauppauge");

	SetDrawingMode(B_OP_COPY);

	// Create Controller window
	fControlWindow = new ControlWindow(fVideoSource);
}

VideoView::~VideoView()
{
	printf("VideoView::~VideoView() - START\n");
	
	// When we are destroyed, we're responsible
	// for taking the video source with us.
	WaitForStop();
	
	delete fVideoSource;
	
	printf("VideoView::~VideoView() - END\n");
}


void VideoView::AttachedToWindow()
{
	// Start the video source going
	fVideoSource->Start();
	
	// Now start the thread that will do the drawing
	resume_thread(drawer_id = spawn_thread(__drawer,"btsk1",B_NORMAL_PRIORITY,this));	

	fControlWindow->Show();
}

void VideoView::Draw(BRect rect)
{
	// We don't bother doing anything in particular for
	// drawing since the thread is taking care of drawing 
	// us once every so many times a second.
	// If we were to do any drawing, it should happen
	// like this:
	//DrawBitmap(bitmap,rect);
}

void
VideoView::WaitForStop()
{
	long   pipo;
	
	printf("VideoView::WaitForStop() - START\n");

	// signal helper threads to stop 
	drawer_quit = 1;
			
	kill_thread(drawer_id);
	wait_for_thread(drawer_id, &pipo);
	printf("drawer dead\n");

	// Tell the video source to stop
	fVideoSource->Stop();

	printf("VideoView::WaitForStop() - END\n");
}



//=======================================================
// Independent drawing thread
//=======================================================
long __drawer(void *arg)
{
	VideoView    *view = (VideoView*)arg;
	
	printf("drawer\n");
	
	while (!drawer_quit) 
	{
		view->Window()->Lock();

		view->DrawBitmap(view->VideoSource()->GetNextFrame(),view->Frame());
	
		view->Window()->Unlock();
		numdrawer++;
	}

	return 0;
}


